.SALL
SUBTTL Define local macros
PAGE
;----------------------------------------------------------------------;
;                                                                      ;
;       SHIFT arithmetic left multiple                                 ;
;                                                                      ;
;----------------------------------------------------------------------;
SHLM    MACRO   REG,CNT

        REPT    CNT                     ; For each shift
        SHL     REG,1                   ; Shift one left
        ENDM

        ENDM
;----------------------------------------------------------------------;
;                                                                      ;
;       SHIFT logical right multiple                                   ;
;                                                                      ;
;----------------------------------------------------------------------;
SHRM    MACRO   REG,CNT

        REPT    CNT                     ; For each shift
        SHR     REG,1                   ; Shift one right
        ENDM

        ENDM
        PAGE
;----------------------------------------------------------------------;
;                                                                      ;
;       Convert BCD to binary                                          ;
;                                                                      ;
;----------------------------------------------------------------------;
BCD_BIN MACRO   DST,SRC                 ; Convert value from BCD to binary

        MOV     AH,SRC                  ; Load source data
        MOV     DST,SRC                 ; Load output initially
        AND     DST,0Fh                 ; Strip high order nibble
        SHRM    AH,4                    ; Get high order nibble
        SHL     AH,1                    ; Multiply by 2
        ADD     DST,AH                  ; Add to the result
        SHLM    AH,2                    ; Multiply by 4
        ADD     DST,AH                  ; Generate final result sum

        ENDM
;----------------------------------------------------------------------;
;                                                                      ;
;       Convert binary to BCD byte                                     ;
;                                                                      ;
;----------------------------------------------------------------------;
BIN_BCD MACRO   DST,SRC                 ; Convert value from binary to BCD

        PUSH    AX                      ; Save registers
        PUSH    DX                      ;
        MOV     AL,SRC                  ; Load source data
        XOR     DX,DX                   ; Clear upper dividend bits
        MOV     AH,DH                   ; Clear lower dividend bits
        MOV     BX,10                   ; Load divisor
        DIV     BX                      ; Divide AX by DX
        SHLM    AX,4                    ; Move divisor to high nibble
        OR      AL,DL                   ; ... and combine with lower nibble
        MOV     DST,AL                  ; Store resulting BCD value
        POP     DX                      ; Restore registers
        POP     AX                      ;

	ENDM

	PAGE
;----------------------------------------------------------------------;
;                                                                      ;
;       PRINT byte/string to screen                                    ;
;                                                                      ;
;----------------------------------------------------------------------;
PRINT   MACRO   VALUE,STRING            ; Print value or DL contents
        LOCAL   LBL                     ; Text label
        LOCAL   SKIP                    ; Skip label

        IFNB    <VALUE>                 ; If register or value specified
        MOV     DL,VALUE                ; Load byte value
        ENDIF

        IFNB    <STRING>                ; If string print operation

        JMP     SHORT SKIP              ; Skip the text
LBL:    DB      STRING,'$'              ; Define the text

SKIP:   PUSH    AX                      ; Save registers
        PUSH    DX                      ;
        LEA     DX,LBL                  ; Get string offset
        MOV     AH,9                    ; Function:  print string
        INT     21h                     ; Request DOS service
        POP     DX                      ; Restore registers
        POP     AX                      ;

        ELSE                            ; Else if single byte

        MOV     AH,2                    ; Select DOS print function
        INT     21h                     ; Print the byte

        ENDIF                           ; Terminate conditional

        ENDM
        PAGE
;----------------------------------------------------------------------;
;                                                                      ;
;       PUSHALL registers                                              ;
;                                                                      ;
;----------------------------------------------------------------------;
PUSHALL MACRO

        IRP     REG,<AX,BX,CX,DX,BP,SI,DI,DS,ES>
        PUSH    REG                     ; Push all registers
        ENDM                            ;
        PUSHF                           ; ... and PSW flags
        CLD                             ; Assume forward string search

        ENDM
;----------------------------------------------------------------------;
;                                                                      ;
;       POPALL registers                                               ;
;                                                                      ;
;----------------------------------------------------------------------;
POPALL  MACRO

        POPF                            ; Restore PSW flags
        IRP     REG,<ES,DS,DI,SI,BP,DX,CX,BX,AX>
        POP     REG                     ; Pop all registers
        ENDM                            ;

        ENDM
;----------------------------------------------------------------------;
;                                                                      ;
;       Define repeat loop count and entrypoint                        ;
;                                                                      ;
;----------------------------------------------------------------------;
REPEAT  MACRO   LBL,CNT

        IFNB    <CNT>                   ; If count specified
        MOV     CX,CNT                  ; Load loop counter
        ENDIF                           ; Otherwise assume already loaded

LBL     LABEL   NEAR                    ; ... and define loop start
        PUSH    CX                      ; Save loop index counter

        ENDM
;----------------------------------------------------------------------;
;                                                                      ;
;       Define repeat loop termination exitpoint                       ;
;                                                                      ;
;----------------------------------------------------------------------;
ENDR    MACRO   LBL,X

        POP     CX                      ; Restore loop counter
        LOOP&X  LBL                     ; ... and continue looping

        ENDM
        PAGE
;----------------------------------------------------------------------;
;                                                                      ;
;       Perform multi-way CASE branch                                  ;
;                                                                      ;
;----------------------------------------------------------------------;
CASE    MACRO   SR,BASE,LIMIT,PREFIX,DISPLIST
        LOCAL   SKIP
        LOCAL   VECTBL


        CMP     SR,LIMIT                ; Compare to limit
        JA      SKIP                    ; ... and skip if case invalid
        SUB     SR,BASE                 ; Subtract base
        JB      SKIP                    ; ... and skip if case invalid

        SAL     SR,1                    ; Convert word to byte offset
        JMP     WORD PTR CS:VECTBL[&SR&]; Vector via table

VECTBL:
        IRP     VECTOR,<DISPLIST>       ; Repeat for each entry
        CASEVEC PREFIX,VECTOR           ; Define case vector
        ENDM                            ; Terminate dispacement list
SKIP:                                   ; Skip here if no cases match


        ENDM
;----------------------------------------------------------------------;
;                                                                      ;
;       Perform multi-way CASE branch                                  ;
;                                                                      ;
;----------------------------------------------------------------------;
CASEVEC MACRO   FAC,LBL

        DW      OFFSET &FAC&LBL&        ; Define case vector

        ENDM
;----------------------------------------------------------------------;
;                                                                      ;
;       Define uppercase conversion macro                              ;
;                                                                      ;
;----------------------------------------------------------------------;
;       Perform uppercase conversion on byte register                  ;
;----------------------------------------------------------------------;
UPCASE  MACRO   REG
        LOCAL   SKIP

        AND     REG,7Fh                 ; Strip parity bit
        CMP     REG,61h                 ; Too small?
        JL      SKIP                    ; Yes, skip conversion
        CMP     REG,7Ah                 ; Too large?
        JG      SKIP                    ; Yes, skip conversion
        AND     REG,5Fh                 ; Perform lower-to-upper conversion
SKIP:                                   ; ... and continue processing

        ENDM

